/*
   File pat_nrg_new.c - patch "El Torito"-image in order to work
   with FSC ACTIVY 3XX-Series. 

   Written by Claus Oberste-Brandenburg (2005).

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#include "config.h"
#include "mkisofs.h"
#include "iso9660.h"

#define BUFSIZE SECTOR_SIZE*512  // 2024 * 512 = 1M

char buffer[BUFSIZE];
static struct eltorito_validation_entry valid_desc;

/*
 *  Taken from write.c of mkhybrid-package
 */
void FDECL2(set_721, char *, pnt, unsigned int, i)
{
     pnt[0] = i & 0xff;
     pnt[1] = (i >> 8) &  0xff;
}


/*
 *  Compute the checksum 
 */
int calc_checksum(struct eltorito_validation_entry *test_desc) {
  unsigned int i;
  int				checksum;
  unsigned char		      * checksum_ptr;
  
  checksum=0;
  checksum_ptr = (unsigned char *) test_desc;
  for (i=0; i<sizeof(valid_desc); i+=2) 
    {
      checksum += (unsigned int)checksum_ptr[i];
      checksum += ((unsigned int)checksum_ptr[i+1])*256;
    }
  return(checksum);
} 


/*
 * Patch the buffer 
 */
int patch_buf(char *buf, int bufsize) {
  unsigned int i,j;
  char currid[25];
  int checksum,*curr_word;
  int patched_buf = 0;

 struct  eltorito_validation_entry *curr_entry;
  
  for (i=0;i<bufsize-sizeof(valid_desc);i++) {
    curr_entry=(struct eltorito_validation_entry*)&buf[i];
    if(curr_entry->key1[0] == 0x55 && 
       curr_entry->key2[0] == (char)0xAA &&
       curr_entry->headerid[0] == 0x01 &&  
       curr_entry->arch[0] == EL_TORITO_ARCH_x86 ) {

      checksum = calc_checksum(curr_entry);   
      
      if((checksum & 0x0000FFFF) == 0 ) {

	memcpy(currid,curr_entry->id,24);
	printf("Found Validation ID, String: %s\n",currid);

	set_721(curr_entry->cksum,0);
	memset(curr_entry->id,0,24);
 	memcpy(curr_entry->id,"ACTIVY\0",7);

	checksum = -calc_checksum(curr_entry);   
	set_721(curr_entry->cksum, (unsigned int) checksum);
	patched_buf = 1;
      }
    }
  }

  return(patched_buf);


}


/*
 * Patch files
 */
int  patch_files(char *infile, char *outfile) {
  FILE *fh_in, *fh_out;
  int buf_fill,bytes_read,bytes_written;
  int patched_file = 0;
  
  fh_in = fopen(infile,"rb");
  if(!fh_in) {
    printf("Could not open file %s for reading\n",infile);
    exit (1);
  }

  fh_out = fopen(outfile,"wb");
  if(!fh_out) {
    printf("Could not open file %s for writing\n",outfile);
    close(fh_in);
    exit (1);
  }

  bytes_read = 0;
  bytes_written = 0;
  do {
    buf_fill = fread(buffer,1,BUFSIZE,fh_in);
    bytes_read = bytes_read + buf_fill;

    if(patched_file==0) {
      patched_file += patch_buf(buffer,buf_fill);
    }

    bytes_written = bytes_written + fwrite(buffer,1,buf_fill,fh_out);
    
  } while (buf_fill == BUFSIZE);
  /*
    printf(" read %i, wrote %i Bytes\n",bytes_read,bytes_written);
  */

  close(fh_in);
  close(fh_out);

  return(patched_file);
}


/*
 *  MAIN
 */
int main(int argc, char *argv[]){
  
  if(argc != 3) {
    printf("Usage: %s image_in image_out\n",argv[0]);
    exit (1);
  }

  if(patch_files(argv[1],argv[2])){
    exit(0);
  } 
  else {
    printf("DID NOT FIND VALIDATION-ENTRY!\n");
    exit(1);
  }
}
